#-*- coding: utf-8 -*-

from z3c.form import interfaces

from zope import schema
from zope.interface import Interface

from zope.i18nmessageid import MessageFactory

_ = MessageFactory('ityou.thumbnails')


class IThumbnails(Interface):
    """Marker Interface
    """
    
class IThumbnailSettings(Interface):
    """Global thumbnail settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    cutycapt_installed = schema.Bool(
            title=_(u"CutyCapt for Website-Thumbnails installed?"),
            description=_(u"If CutyCapt is installed, nice thumbnails of Websites are possible."),
            required=False,
            default=False,
        )


# -*- coding: utf-8 -*-
"""Define a browser view for the Archive Text content type. In the FTI 
configured in profiles/default/types/*.xml, this is being set as the default
view of that content type.
"""
from Acquisition import aq_inner
from zope.interface import implements
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from ZTUtils import Batch
from Products.CMFDefault.browser.utils import ViewBase
from plone.app.content.browser.foldercontents import FolderContentsTable
from zope.component.hooks import getSite

from zope.component import getMultiAdapter

from datetime import datetime, date
from . import isProductAvailable
from ..thumbnail import ThumbnailManager

tm = ThumbnailManager()

class ThumbnailListingView(BrowserView):
    """Listing view with thumbnails and portal actions
    """
    
    html_template = ViewPageTemplateFile('templates/thumbnail_listing.pt')

    def __call__(self):
        """
        Returns Template
        """
        context = aq_inner(self.context)
        context.REQUEST.set('action', 'thumbnail_listing')
        mt = getToolByName(context, "portal_membership")

        self.DRAGDROP_INSTALLED = isProductAvailable("ityou.dragdrop")
        self.ESI_ROOT = getSite().absolute_url()        

        return self.html_template()
   
    def getFolderContents(self):
        """
        Gets the folder contents and returns them and folder actions.
        Thumbnails come from getThumbnail function of class ThumbnailManager
        returns 
        {
            'id':             'gaga',
            'title':          'GAGA',
            'description':    'This is gaga.',
            'url':            'http://...',          
            'path':           '/inet/inet_documents/...', 
            'thumbnail_url':   'http://.../inet/inet_documents/gaga/@@show_thumbnail', 
            'type':            'document',
            'author':            'Hans Mustermann',
            'created':        '08/05/2013',
            'edited':        '09/05/2013',
            'icon':        Icon of Document Type,
            'content_type_class':    contenttype-folder
             "timestamp_mod"        : obj.created(),
             "timestamp_create"     : obj.modification_date,
        }, 

        """
        context = aq_inner(self.context)
        plone = getMultiAdapter((self.context, self.request), name="plone")
        catalog = getToolByName(context, 'portal_catalog')
        mt = getToolByName(self, 'portal_membership')
        
        folder_path = '/'.join(context.getPhysicalPath())
        brains = catalog(path={'query': folder_path, 'depth': 1}, sort_on='getObjPositionInParent')

        items = []
        for brain in brains:
            obj = brain.getObject()
            author_id = obj.Creator()
            author = mt.getMemberById(author_id)
            if author:
                author_name = author.getProperty("fullname")
            else:
                author_name = author_id
            
            contenttype = obj.portal_type
            contenttype_class = "contenttype-" + contenttype.lower()

            items.append({
                                 "title"            : obj.Title(),
                                 "description"      : obj.Description(),
                                 "url"              : obj.absolute_url(),
                                 "path"             : brain.getURL(relative=True),
                                 "id"               : obj.getId(),
                                 "type"             : brain.Type,
                                 "uid"              : obj.UID(),
                                 "author"           : author_name,
                                 "created"          : context.toLocalizedTime(obj.created()),
                                 "edited"           : context.toLocalizedTime(obj.modification_date),
                                 "icon"             : obj.getIcon(),
                                 "contenttype_class": contenttype_class,
                                 "timestamp_create"        : int(obj.created().timeTime()),
                                 "timestamp_mod"     : int(obj.modification_date.timeTime()),
                                 })
        return { "folder_items" : items}
    
    def canDelete(self):
        """ Checks if user can delete items in current folder
        """
        
    






    



# -*- coding: utf-8 -*-
import os

from Acquisition import aq_inner
from zope.interface import implements
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from zope.component.hooks import getSite

from .. import THUMBNAIL_HOME, INSTANCE_PATH
from . import DEFAULT_IMAGES_PATH

class ShowThumbnail(BrowserView):
    """
    """
    def __call__(self):
        """
        If a thumbnail of the document (context) exists, this function returns the image,
        it simulates an URL.
        If there's no thumbnail, the function returns None.
        """
        context = aq_inner(self.context)
        size = context.REQUEST.get("size")
        if not size:
            size = ""
        else:
            size = "_" + size
        response = context.REQUEST.RESPONSE
       
        uid = context.UID()
        type = context.portal_type

        tn_uri = THUMBNAIL_HOME + '/' + uid[0:2] + '/' + uid + size + '.png'

        if os.path.isfile(tn_uri):
            f = open(tn_uri, 'r')
            tn = f.read()    
            f.close()
        else:
            if type == 'Folder':                
                thumbnail_path = DEFAULT_IMAGES_PATH + 'folder-thumbnail.png'                 
            else:
                thumbnail_path = DEFAULT_IMAGES_PATH + 'no-thumbnail.png'
            tnf = open(thumbnail_path, 'r')
            tn = tnf.read()
            tnf.close()     
            
        response.setHeader('Content-Type','image/png')
        return tn


# -*- coding:utf-8 -*-
from zope.interface import implements
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.viewlet.interfaces import IViewlet

from Acquisition import aq_inner
from Products.Five.browser import BrowserView

class ThumbnailViewlet(BrowserView):
    """Viewlet used to display the buttons
    """
    implements(IViewlet)
    render = ViewPageTemplateFile("templates/thumbnail.pt")

    def __init__(self, context, request, view=None, manager=None):
        super(ThumbnailViewlet, self).__init__(context, request)
        self.context = context


# -*- coding: utf-8 -*-
import os
import urllib2
from datetime import datetime, date
import magic
import re

from time import time

from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from ityou.thumbnails.interfaces import IThumbnailSettings

from Acquisition import aq_inner
from zope.interface import implements
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from zope.component.hooks import getSite

from . import THUMBNAIL_HOME, INSTANCE_PATH, TMP_FOLDER_LOCATION, HTML_TMPL
from . import THUMBNAIL_SUFFIX, LARGE_THUMBNAIL_SUFFIX, LISTING_THUMBNAIL_SUFFIX, CONVERTABLE_DOCTYPES

from browser.show_thumbnail import ShowThumbnail

import logging

try:
    os.mkdir(THUMBNAIL_HOME)
except OSError:
    logging.info("%s already exists. No need to create." % THUMBNAIL_HOME)

try:
    os.mkdir(TMP_FOLDER_LOCATION)
except OSError:
    logging.info("%s already exists. No need to create." % TMP_FOLDER_LOCATION)

class ThumbnailManager():
    """
    The thumbnail manager manages the thumbnails. The functions of the class generates them
    and deliver them to templates 
    """
    
    def getThumbnail(self, obj, size = ""):
        """
        This function checks if the document has already a new thumbnail or if a thumbnail
        has to be generated. If not it returns the URL of the thumbnail (link to show_thumbnail view
        on document). If a thumbnail has to be generated it calls the _createTmpFile() function.
        """
        obj_uid = obj.UID()
        tmp_file = None
                 
        if obj.portal_type == "Image":
            if not size:
                return obj.absolute_url() + '/image_thumb'
            elif size == "large":
                return obj.absolute_url() + '/image_large'
            elif size == "listing":
                return obj.absolute_url() + "/image_thumb"

        tn_path  = THUMBNAIL_HOME + '/' + obj_uid[0:2] + '/'
        tn       = tn_path + obj_uid + THUMBNAIL_SUFFIX[1]
        tn_large = tn_path + obj_uid + LARGE_THUMBNAIL_SUFFIX[1]
        tn_listing = tn_path + obj_uid + LISTING_THUMBNAIL_SUFFIX[1]
        
        obj_modified = obj.modification_date.millis() / 1000
        
        if not os.path.isfile(tn) or not os.path.isfile(tn_large):
            tmp_file = self._createTmpFile(obj, tn)
            if tmp_file:
                if not os.path.isfile(tn):
                    self._createThumbnail(tmp_file, tn_path, tn, size=THUMBNAIL_SUFFIX[0])
                if not os.path.isfile(tn_large):
                    self._createThumbnail(tmp_file, tn_path, tn_large, size=LARGE_THUMBNAIL_SUFFIX[0])
                if not os.path.isfile(tn_listing):
                    self._createThumbnail(tmp_file, tn_path, tn_listing, size=LISTING_THUMBNAIL_SUFFIX[0])
        
        if os.path.isfile(tn):                
            if obj_modified > int(os.path.getmtime(tn)):
                tmp_file = self._createTmpFile(obj, tn)
                if tmp_file:
                    self._createThumbnail(tmp_file, tn_path, tn, size=THUMBNAIL_SUFFIX[0])
                    self._createThumbnail(tmp_file, tn_path, tn_large, size=LARGE_THUMBNAIL_SUFFIX[0])
                    self._createThumbnail(tmp_file, tn_path, tn_listing, size=LISTING_THUMBNAIL_SUFFIX[0])
            
        if tmp_file:
            self._removeFile(tmp_file)
        
        return obj.absolute_url() + '/@@show_thumbnail?size=' + size
    
    @memoize
    def getCachedThumbnail(self, uid, size = ""):
        """ Get Thumbnail in cached version
        """
        return self.getThumbnail(uid, size)
    
    def _removeFile(self,tmp_file):
        """Removes a file form the filesystem
        """
        os.system('rm -f %s' % tmp_file)
        return 

    def _createTmpFile(self, obj, tn):
        """Function creates a html file or other file which ImageMagic 
        can convert. The Thumbnail is saved in /var/thumbnails/...
        """
        obj_uid = obj.UID()
        doc_type = obj.portal_type

        tf = TMP_FOLDER_LOCATION + '/' + obj_uid
        #try:
        if doc_type == "File":
            doc_type = obj.getFilename().split('.')[-1]
            tmp_file = tf + '.' + doc_type 
            f = open(tmp_file, 'w')            
            input = obj.getFile()
            f.write(str(input))
            f.close()
            try:
                # if unoconv is running:
                if doc_type in CONVERTABLE_DOCTYPES:
                    res = os.system("unoconv -f pdf %s" % tmp_file)
                    os.system(  'rm -f %s' % tmp_file  )
                    tmp_file = tf + '.pdf'
            except:
                logging.info('uniconv is not running - no thumbnail of "%s" documents can be generated' % obj_uid)

        elif doc_type == "Link":
            # first check if url is valid!
            if not self._is_valid_url(obj.getRemoteUrl()):
                logging.warn('URL "%s" is not valid!' % obj.getRemoteUrl() )
                return

            tmp_file = tf + ".png"
            f = open(tmp_file, 'w')
            if getUtility(IRegistry).forInterface(IThumbnailSettings).cutycapt_installed: 
                tmp_file_to_crop = INSTANCE_PATH + '/var/tmp/' + obj_uid + 'CROP.png'
                os.system(  "xvfb-run --auto-servernum --server-args='-screen 0, 800x600x24' /usr/local/bin/CutyCapt --url=%s --out=%s" % (obj.getRemoteUrl(), tmp_file_to_crop) )
                os.system(  'convert "%s" -crop 1000x1333+0+0 +repage "%s"' % (tmp_file_to_crop, tmp_file)  )
                os.system(  'rm -f %s' % tmp_file_to_crop  )
            else:
                url_site = urllib2.urlopen(obj.getRemoteUrl())
                input = url_site.read()
                html_input = input[str(input).find("<html"):-1]
                f.write(str(html_input))
                f.close()

        elif doc_type in ["Document","Event", "News Item"]:
            tmp_file = tf + ".html"
            f = open(tmp_file, 'w')            

            title = obj.title_or_id().decode('utf-8').encode('iso-8859-15', 'ignore')
            text  = obj.getText().decode('utf-8').encode('iso-8859-15', 'ignore')
            description = obj.Description().decode('utf-8').encode('iso-8859-15', 'ignore')

            if doc_type == "Event":
                event_content = "\n" #TODO Event-Felder !iso-8859-15 
                text += event_content
            elif doc_type == "News Item":
                news_content = "\n" # TODO News-Felder !iso-8859-15 
                text += news_content

            data =  HTML_TMPL % (title, title, description, text)
            f.write(data)
            f.close()
        else:
            tmp_file = None
        
        return tmp_file
        #except:
        #    logging.warning("Warning: Could not create tmp_file for %s" % obj_uid)

    
    def _createThumbnail(self, tmp_file, tn_path, tn, size=THUMBNAIL_SUFFIX[0]):
        """Creates a thumnail
        tmp_file: Location of the temporary file
        tn: Location of the generated thumbnail
        """
        os.system('mkdir -p %s' % tn_path)
        
        try:
            if os.path.isfile(tmp_file):
                logging.info("Create Thumbnail (Path): %s", tn)
                os.system('convert -thumbnail %s "%s"[0] "%s"' % (size, tmp_file, tn))
        except:
            logging.warning("Warning: Could not create thumbnail for %s" % tn)        

        return

    def _is_valid_url(self, url):
        regex = re.compile(
            r'^https?://'  # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'  # domain...
            r'localhost|'  # localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
            r'(?::\d+)?'  # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)

        return url is not None and regex.search(url)






# -*- coding: utf-8 -*-
import json

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from ..thumbnail import ThumbnailManager

tm = ThumbnailManager()

class AjaxGetThumbnail(BrowserView):
    """ Returns Thumbnail after possible generation of it 
    """
    def __call__(self):
        """ Calls getThumbnail function and returns as json
        """
        context = aq_inner(self.context)
        size = context.REQUEST.get("size")
        caching = context.REQUEST.get("caching") or False
        if not size:
            size = ""
        if caching:
            thumb = tm.getCachedThumbnail(self.context, size = size)
        else:
            thumb = tm.getThumbnail(self.context, size = size)
        return json.dumps(thumb)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from zope.interface import implements
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from zope.component.hooks import getSite

class ShowPreview(BrowserView):
    """
    """
    
    html_template = ViewPageTemplateFile('templates/show_preview.pt')
    
    def __call__(self):
        """
        """
        context = aq_inner(self.context)
        mt = getToolByName(context, "portal_membership")
        self.edit_right = mt.checkPermission('edit', context)
        return self.html_template()


# -*- coding: utf-8 -*-

INSTANCE_PATH       = '/'.join(INSTANCE_HOME.split('/')[:-2])
THUMBNAIL_HOME      = INSTANCE_PATH + '/var/thumbnails'
TMP_FOLDER_LOCATION = INSTANCE_PATH + "/var/tmp"
LISTING_THUMBNAIL_SUFFIX         = ['48x48', '_listing.png']
THUMBNAIL_SUFFIX         = ['128x192', '.png']
#LARGE_THUMBNAIL_SUFFIX   = ['512x768', '_large.png']
LARGE_THUMBNAIL_SUFFIX   = ['458x687', '_large.png']

CONVERTABLE_DOCTYPES = ['doc','docx', 'dot', 'xls', 'xlsx', 'ppt', 'pptx','ppa', 'pps', 'xlb', 'xlt',
                        'odt','odp', 'odc','odg','otp','odm','ott','oth','ods',
                        'sdt','sdp', 'sdc',
                        'sxt','sxp', 'sxc','sxd',
                        'txt', 'csv', 
                        'py', 'php', 'js','json'
                        'htm', 'html','rtf','xml',
                        'psd',
                        'pgp',
                        ]
      
HTML_TMPL           = """
<!DOCTYPE html PUBLIC
  "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
    <head>
        <title>%s</title>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    </head>
    <body>
        <h1>%s</h1>
        <div><b>%s</b></div>
        <div id="content">%s</div>
    </body>
</html>
"""


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from zope.component.hooks import getSite
import ityou.thumbnails as me
PRODUCT_PATH = me.__path__[0]
DEFAULT_IMAGES_PATH = PRODUCT_PATH + '/browser/images/'

def isProductAvailable(product):
    """Check if a product is installed and return True, 
    else return False
    """
    qui = getSite().portal_url.getPortalObject().portal_quickinstaller
    installed_products = qui.listInstalledProducts()
    for prod in installed_products:
        if prod["id"] == product and prod["status"] == "installed":
            return True
    return False


#-*- coding: utf-8 -*-

from plone.app.registry.browser import controlpanel

from ityou.thumbnails.interfaces import IThumbnailSettings, _

class ThumbnailSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IThumbnailSettings
    label = _(u"Thumbnail settings")
    description = _(u"""Settings of thumbnail variables
    """)

    def updateFields(self):
        super(ThumbnailSettingsEditForm, self).updateFields()


    def updateWidgets(self):
        super(ThumbnailSettingsEditForm, self).updateWidgets()

class ThumbnailSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = ThumbnailSettingsEditForm